const path = require('path')
const fs = require('fs')
const posthtml = require('posthtml')

const preprocessPhp = require('./preprocess-php')
const processBem = require('./process-bem')
const processNode = require('./process-node')

module.exports = function (source) {
  let php = true
  if (this.query.php !== undefined) {
    php = this.query.php
  }

  const loadTemplate = (filePath) => {
    this.dependency(filePath)
    return fs.readFileSync(filePath, 'utf-8')
  }

  if (php) {
    source = '<?$params = (object) array();?>' + source
    source = preprocessPhp(source)
  }

  source = posthtml()
    .use((tree) => {
      tree.match({}, (node) => {
        return processNode(
          node, {
            currentRoot: path.dirname(this.resourcePath),
            rootDir: this.query.root,
            loadTemplate,
            php,
          })
      })
      return tree
    })
    .use((tree) => {
      tree.match({}, (node) => {
        processBem(node)
        return node
      })
      return tree
    })
    .process(source, {sync: true})
    .html

  if (php) {
    // unescape php
    source = source
      .replace(/___php_begin___(.*?)___php_end___/g,
        function (_, php) {
          return '<?' + new Buffer(php, 'base64').toString('utf8') + '?>'
        })
  }

  return source
}
