const path = require('path')

const posthtmlApi = require('posthtml/lib/api')
const parseToPostHtml = require('posthtml-parser')

const preprocessPhp = require('./preprocess-php')

const processNode = require('./process-node')

module.exports = function (node, opts) {
  const {
    currentRoot,
    rootDir,
    loadTemplate,
  } = opts

  if (!node.tag || node.tag !== 'extends') {
    return node
  }
  if (!node.attrs.src || !node.content || !Array.isArray(node.content)) {
    return node
  }
  let mix
  if (node.attrs.mix) {
    mix = node.attrs.mix
  }
  let mods
  if (node.attrs.mods) {
    mods = node.attrs.mods
  }

  const blocks = {}
  for (let i = 0; i < node.content.length; ++i) {
    const children = node.content[i]
    if (typeof children !== 'object') {
      continue
    }
    if (children.tag !== 'block') {
      return node
    }
    if (!children.attrs || !children.attrs.name) {
      return node
    }
    blocks[children.attrs.name] = children.content
  }

  let filePath = node.attrs.src
  if (path.isAbsolute(filePath)) {
    filePath = path.resolve(__dirname, rootDir) + filePath
  } else {
    filePath = currentRoot + '/' + filePath
  }
  let content = loadTemplate(filePath)
  content = preprocessPhp(content)
  const layoutTree = parseToPostHtml(content)
  posthtmlApi.match.call(layoutTree, {}, (node) => {
    if (node.tag === 'extends') {
      if (mods != null) {
        if (node.attrs.mods != null) {
          node.attrs.mods += ',' + mods
        } else {
          node.attrs.mods = mods
        }
      }
    } else if (node.tag !== 'include') {
      if (node.attrs && node.attrs.extends != null) {
        if (node.attrs.class) {
          node.attrs.class += ' ' + mix
        } else {
          node.attrs.class = mix
        }
        if (node.attrs.mods != null) {
          node.attrs.mods += ',' + mods
        } else {
          node.attrs.mods = mods
        }
        delete node.attrs.extends
      }
      if (node.attrs && node.attrs.mix != null) {
        node.attrs.block = mix + '_' + node.attrs.mix
        delete node.attrs.mix
      }
    }
    return processNode(node, Object.assign({}, opts, {
      currentRoot: path.dirname(filePath),
    }))
  })

  posthtmlApi.match.call(layoutTree, {}, (node) => {
    if (node.skip) {
      delete node.skip
      return node
    }

    if (node.tag === 'block' && node.attrs && node.attrs.name) {
      const replaceNode = blocks[node.attrs.name]
      if (!replaceNode) {
        return
      }
      delete blocks[node.attrs.name]
      posthtmlApi.match.call(replaceNode, {}, (node) => {
        node.skip = true
        return node
      })
      return {
        tag: false,
        content: replaceNode,
      }
    }

    return node
  })

  return layoutTree
}
