const path = require('path')

const posthtmlApi = require('posthtml/lib/api')
const parseToPostHtml = require('posthtml-parser')

const preprocessPhp = require('./preprocess-php')

const processNode = require('./process-node')

module.exports = function (node, opts) {
  const {
    currentRoot,
    rootDir,
    loadTemplate,
    php,
  } = opts

  if (!node.tag || node.tag !== 'include') {
    return node
  }
  if (!node.attrs.src) {
    return node
  }
  let mix
  if (node.attrs.mix) {
    mix = node.attrs.mix
  }

  let filePath = node.attrs.src
  if (path.isAbsolute(filePath)) {
    filePath = path.resolve(__dirname, rootDir) + filePath
  } else {
    filePath = currentRoot + '/' + filePath
  }
  let content = loadTemplate(filePath)

  if (php) {
    content = `\
<?\
$___args = $params;\
$params = (object) array();\
call_user_func(function () use(&$___args) {\
extract((array) $___args);\
$inherit = &$___args;\
unset($___args);\
$params = (object) array();\
?>\
${content}\
<?\
});\
?>
`
    content = preprocessPhp(content)
  }

  const layoutTree = parseToPostHtml(content)
  posthtmlApi.match.call(layoutTree, {}, (node) => {
    if (node.attrs && node.attrs.component) {
      node.attrs.elem = mix
      node.attrs.block = node.attrs.component
      delete node.attrs.component
    }
    return processNode(node, Object.assign({}, opts, {
      currentRoot: path.dirname(filePath),
    }))
  })
  return {
    tag: false,
    content: layoutTree,
  }
}
